package fake_client;

use Socket;
use strict;

&check_env_vars;
&main;

sub check_env_vars 
{
   my $chkresult = 1;

   print "Checking Environment Variables ...\n";
   my @envvars = qw/ HPCALE_FTPD_HOST
                     HPCALE_FTPD_PORT
                     HPCALE_CLIENT_SSLCERT /;


   foreach(@envvars){
      if(chk_envvar($_) < 0){
         $chkresult = -1;
      }
   }

   if($chkresult < 0){
      print "Missing Environment Variable.\n";
      exit;
   }
   else{
      print "All necessary environment variables are defined.\n";
   }
}

sub chk_envvar {
   my $var = shift;
   if(!defined($ENV{$_})){
      print "Undefined environment variable: $var\n";
      return -1;
   }
   else{
      print "  Checking for $var....yes\n";
      return 1;
   }
}


sub main {
   my $host = $ENV{HPCALE_FTPD_HOST};
   my $port = $ENV{HPCALE_FTPD_PORT};
   my $cert = $ENV{HPCALE_CLIENT_SSLCERT};

   my $workdir = "/export/home/iyoon/repsvn_src/HPCALE/client/xjdexample";
   my $xjdprefix = "mpibug-mpibug";

   # connection and certificate check
   my $r = contact_ftpdaemon($host,$port,$cert);
   if ($r <= 0) {
      return -1;
   }

   # get file list to send
   my $r = get_files($workdir, $xjdprefix);
   if (not defined $r ) {
      return -1;
   }

   # send the number of files to send
   my @files = @{$r};
   send(SOCK, "$#files.", 0);

   # send actual files
   foreach(@files){
      send_file($_->[0], $_->[1]);
   }

   terminate_connection();
}

sub pack_line {
   my $str = shift;

   chomp $str;
   $str =~ s/^\s+//;
   $str =~ s/\s+$//;
   return $str;
}

sub get_files {
   my ($wd, $prefix) = @_;
   my @files = ();

   push @files, ["als.bib", "als.bib"];
   push @files, ["pt99.jpg", "pt99.jpg"];

   if(0){
      return undef;
   }

   return \@files;
}

sub contact_ftpdaemon {
   my ($host, $port, $cert) = @_;

   my $proto  = getprotobyname('tcp');
   my $iaddr = inet_aton($host);
   my $paddr = sockaddr_in($port, $iaddr);

   # create socket
   socket(SOCK, PF_INET, SOCK_STREAM, $proto) or return 0;

   # establish connection 
   connect(SOCK, $paddr) or die return 0;

   # check certificate
   check_cert($cert) or die return 0;

   return 1;
}

sub terminate_connection 
{
   shutdown (SOCK, 1);
}


sub check_cert 
{
   my $cert = shift;
   send_file($cert, "mycert.pem");

   my $result = '';
   recv(SOCK, $result, 1, 0);
   if($result == 1){
      print "<certificate is ok>\n";
      return 1;
   }
   else{
      print "<invalid certificate.>\n";
      return 0;
   }
}

sub send_file 
{
   my ($fn, $tn) = @_;
   my $data = '';

   # send filename length
   send(SOCK, length($tn), 0);
   send(SOCK, '.', 0);
   
   # send destination filename
   send(SOCK, $tn, 0);

   open TOSEND, "< $fn";
   binmode TOSEND;

   # send file size
   my $size = -s TOSEND;

   send(SOCK, $size, 0); # send size
   send(SOCK, '.', 0); # end_size mark

   # send file contents
   my $chunksize = 1000;
   my $quot = $size / $chunksize;
   my $remain = $size % $chunksize;

   my $i = 1;
   my $sendok = 1;
   while($i <= $quot) {
      if($sendok == 1){
         read(TOSEND, $data, $chunksize);
      }

      my $realsent = send(SOCK, $data, 0);  

      recv(SOCK, $data, 1, 0);
      if($data == 1){
         $sendok = 1;
         #print "<$i, ok to send>\n";
         $data = ''; 
      }
      else{
         $sendok = 0;
      }
      $i++;
   }
   
   $data = '';
   read(TOSEND, $data, $remain);
   send(SOCK, $data, 0);
   # end of file contents transfer

   close TOSEND;
}


