package ftpdaemon;

use util::debug;
use util::utilstr;
use util::utilerr;
use util::utilhash;
use util::dirman;
use ftpcomm;
use strict;
use warnings;

my $mod  = "HPCALE FTP Daemon Main";
my $lpipe = undef;
my $epipe = undef;

check_env_vars();
set_env();
ftpcomm::init_server();   

sub check_env_vars 
{
   my $chkresult = 1;

   debug::logmsg($mod, "STDOUT", "Checking Environment Variables ...");
   my @envvars = qw/ HPCALE_FTPD_HOST
                     HPCALE_FTPD_PORT
                     OPENSSL_BIN
                     HPCALE_FTPD_CADIR
                     HPCALE_FTPD_CAFILE
                     HPCALE_FTPD_HOME
                     HPCALE_FTPD_LOGPATH
                     HPCALE_FTPD_LOGFILE
                     HPCALE_REMOTE_JOBDIR
                     HPCALE_DEBUG_LEVEL /;

   foreach(@envvars){
      if(chk_envvar($_) < 0){
         $chkresult = -1;
      }
   }

   if($chkresult < 0){
      debug::logmsg($mod, "STDOUT", "Missing Environment Variable.");
      exit;
   }
   else{
      debug::logmsg($mod, "STDOUT", "All necessary environment variables are defined.");
   }
}

sub chk_envvar 
{
   my $var = shift;
   if(!defined($ENV{$_})){
      debug::logmsg($mod, "STDERR", "Undefined environment variable: $var");
      return -1;
   }
   else{
      debug::logmsg($mod, "STDERR", "  Checking for $var....yes");
      return 1;
   }
}

sub set_env 
{
   $lpipe = "$ENV{HPCALE_FTPD_LOGPATH}/$ENV{HPCALE_FTPD_LOGFILE}";
   $epipe = "$ENV{HPCALE_FTPD_LOGPATH}/$ENV{HPCALE_FTPD_LOGFILE}";

   ftpcomm::set_env($lpipe, $epipe);
   utilstr::set_env($lpipe, $epipe);
   utilhash::set_env($lpipe, $epipe);
   utilerr::set_env($lpipe, $epipe);
   dirman::set_env($lpipe, $epipe);
   debug::set_env();
}

