# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package dirman;

use strict;

my $mod = "dirman";
my $lpipe = undef;
my $epipe = undef;

my $ERR000 = "Directory is created.";
my $ERR001 = "Directory alreay exists.";
my $ERR002 = "Host is unavailable.";

sub set_env {
   ($lpipe,$epipe) = @_;
}

sub create_remote_dir
{
   my ($host, $path, $jobid) = @_;

   if(isHostAlive($host) < 0){
      debug::errmsg($mod, $epipe, $ERR002);
      return -1;
   }

   if($path =~ /^\//){
      debug::dbgmsg(3, $mod, $lpipe, "absolute path: $path");

      return &create_absolute_path($host, $path);
   }
   else{
      debug::dbgmsg(3, $mod, $lpipe, "relative path: $path");
      debug::dbgmsg(3, $mod, $lpipe, "remote job dir: $ENV{HPCALE_REMOTE_JOBDIR}");
      debug::dbgmsg(3, $mod, $lpipe, "jobid: $jobid");
     
      return &create_relative_path($host, $path, $jobid); 
   }
}

sub create_absolute_path
{
   my ($host, $path) = @_;

   if(&dirExist($host, $path) == 1){
      debug::dbgmsg(3, $mod, $lpipe, "Directory <$path> exist");
      return 1;
   }
   else{
      # UNIX mkdir return 0 if it succeed.
      my $rcode = system ("ssh $host 'mkdir -p $path'");
      if($rcode == 0){
         debug::logmsg($mod, $lpipe, "Directory created: <$path>");
         return 1;
      }
      else{
         debug::logmsg($mod, $lpipe, "Directory creation failed: <$path>");
         return 1;
      }
   }
}

sub create_relative_path
{
   my ($host, $path, $jobid) = @_;

   my $jobdir = "$ENV{HPCALE_REMOTE_JOBDIR}/$jobid";
   my $home = &getHomeDir($host);
   
   if(&dirExist($host, "$home/$jobdir/$path") == 1){
      debug::dbgmsg(3, $mod, $lpipe, "Directory <$home/$jobdir/$path> exist");
      return 1;
   }
   else{
      # UNIX mkdir return 0 if it succeed.
      my $rcode = system ("ssh $host 'mkdir -p $home/$jobdir/$path'");
      if($rcode == 0){
         debug::logmsg($mod, $lpipe, "Directory created: <$home/$jobdir/$path>");
         return 1;
      }
      else{
         debug::logmsg($mod, $lpipe, "Directory creation failed: <$home/$jobdir/$path>");
         return -1;
      }
   }
}

sub dirExist {
   my $host = shift;
   my $path = shift;

   # redirect stdout and stderr to /dev/null
   my $dir = system "ssh $host '/bin/sh -c \"ls $path >/dev/null 2>/dev/null\"'";

   debug::dbgmsg(3, $mod, $lpipe, "Checking directory existence....");
  
   if($dir == 256){
      debug::dbgmsg(3, $mod, $lpipe, "directory does not exist");
      return -1;
   }
   elsif($dir == 0){
      debug::dbgmsg(3, $mod, $lpipe, "directory exists");
      return 1;
   }
   else{
      debug::dbgmsg(3, $mod, $lpipe, "unknown return code. ");
      return -1;
   }
}

sub fileExist {
   my $host = shift;
   my $file = shift;

   my $dir = `ssh $host '/bin/sh -c "ls $file 2> /dev/null"'`;

   debug::dbgmsg(3, $mod, $lpipe, "Checking file existence....");
    
   if(length($dir) == 0){
      debug::dbgmsg(3, $mod, $lpipe, "Fils $file does not exist in $host");
      return -1;
   }
   else{
      debug::dbgmsg(3, $mod, $lpipe, "Fils $file exists in $host");
      return 1;
   }
}

sub getHomeDir {
   my $host = shift;

   # get home directory of clusterheads[$i]
   my $home = `ssh $host 'env | grep '^HOME=''`;
   $home =~ /.*=(.*)/;
   $home = $1;
   chomp $home;

   debug::dbgmsg(3, $mod, $lpipe, "Remote Home of $host is $home");
   return $home;
}

sub checkProgramDir {
   my $host = shift;
   my $prog = shift;

   # get home directory of clusterheads[$i]
   my $loc = `ssh $host 'which $prog '`;
   chomp $loc;

   debug::dbgmsg(3, $mod, $lpipe, "Remote location for program $prog is $loc");
   return $loc;
}


sub isHostAlive {
   my $host = shift;

   my $id = `ssh $host 'whoami'`;

   if(length($id) == 0){
      debug::dbgmsg(3, $mod, $lpipe, "Remote host $host is dead.");
      return -1;
   }
   else{
      debug::dbgmsg(3, $mod, $lpipe, "Remote host $host is alive.");
      return 1;
   }
}

1;
