# $Author: iyoon $
# $Date: 2006/11/30 20:22:29 $
# $Revision: 1.2 $
#
# Modified by Il-Chul Yoon <iyoon@cs.umd.edu>
# Extracted from Skoll project at University of Maryland, College Park

eval '(exit $?0)' && eval 'exec perl -S $0 ${1+"$@"}'
    & eval 'exec perl -S $0 $argv:q'
    if 0;

use strict;
use warnings;
use Cwd;
use File::Path;

my ($errors, $warnings) = (0, 0);
my $perl_module_lst_file = "./serverpm.lst";

&check_perl_modules ;
&check_env_vars ;
  
&create_directories ;

&install_webservice ;
&install_server ;
&install_ftpdaemon ; 

print "\nInstallation finished ...\n";

sub check_env_vars {
   my $chkresult = 1;

   print "\nChecking Environment Variables ...\n";
   my @envvars = qw/ APACHE_HOME 
                     HTDOCS_HOME
                     HPCALE_PERL_PATH
                     HPCALE_PM_PATH
                     HPCALE_SERVER_HOST
                     HPCALE_SERVER_DOMAIN
                     HPCALE_SERVER_PORT
                     HPCALE_SVC
                     HPCALE_SERVER_HOME
                     HPCALE_SERVER_PATH
                     HPCALE_SERVER_DB
                     HPCALE_SERVER_DB_HOST
                     HPCALE_SERVER_DB_PORT
                     HPCALE_SERVER_DB_USER
                     HPCALE_SERVER_DB_PASS
                     HPCALE_LOGPATH
                     HPCALE_LOGFILE
                     HPCALE_DEBUG_LEVEL
                     OPENSSL_BIN
                     HPCALE_FTPD_CADIR
                     HPCALE_FTPD_CAFILE
                     HPCALE_FTPD_HOST
                     HPCALE_FTPD_PORT
                     HPCALE_FTPD_HOME
                     HPCALE_FTPD_LOGPATH
                     HPCALE_FTPD_LOGFILE
                     HPCALE_JOBDIR
                     HPCALE_REMOTE_JOBDIR 
                     HPCALE_RESERVE_RETRY /;

   foreach(@envvars){
      if(chk_envvar($_) < 0){
         $chkresult = -1;
      }
   }

   if($chkresult < 0){
      print "Missing Environment Variable.\n";
      exit;
   }
   else{
      print "All necessary environment variables are defined.\n";
   }
}

sub chk_envvar {
   my $var = shift;
   if(!defined($ENV{$_})){
      print STDERR "Undefined environment variable: $var\n";
      return -1;
   }
   else{
      print STDERR "  Checking for $var....yes\n";
      return 1;
   }
}

# Create necessary directories
sub create_directories 
{
   print "\nRemoving previous HPCALE directories ...\n";

   rmtree(["$ENV{HTDOCS_HOME}/HPCALE/sessions"], 1, 1);
   rmtree(["$ENV{APACHE_HOME}/cgi-bin/HPCALE"], 1, 1);
   # intentionally removed.
   #rmtree(["$ENV{HPCALE_SERVER_HOME}"], 1, 1);
   rmtree(["$ENV{HPCALE_FTPD_HOME}"], 1, 1);
   rmtree(["$ENV{HPCALE_LOGPATH}"], 1, 1);
   rmtree(["$ENV{HPCALE_FTPD_LOGPATH}"], 1, 1);
   rmtree(["$ENV{HPCALE_JOBDIR}"], 1, 1);
   
   print "\nCreating HPCALE directories ...\n";

   mkpath(["$ENV{HTDOCS_HOME}/HPCALE/sessions"], 1, 0711);
   mkpath(["$ENV{APACHE_HOME}/cgi-bin/HPCALE"], 1, 0711);
   mkpath(["$ENV{HPCALE_SERVER_HOME}"], 1, 0711);
   mkpath(["$ENV{HPCALE_FTPD_HOME}"], 1, 0711);
   mkpath(["$ENV{HPCALE_LOGPATH}"], 1, 0711);
   mkpath(["$ENV{HPCALE_FTPD_LOGPATH}"], 1, 0711);
   mkpath(["$ENV{HPCALE_JOBDIR}"], 1, 0711);
}


sub install_webservice {
   open PATHOUT, "> $ENV{APACHE_HOME}/cgi-bin/HPCALE/.hpcale-libpath";
   print PATHOUT "$ENV{HPCALE_SERVER_PATH}\n";
   print PATHOUT "$ENV{HPCALE_PM_PATH}\n";
   print PATHOUT "$ENV{HPCALE_LOGPATH}\n";
   print PATHOUT "$ENV{HPCALE_LOGFILE}\n";
   print PATHOUT "$ENV{HPCALE_DEBUG_LEVEL}\n";
   print PATHOUT "$ENV{HPCALE_JOBDIR}\n";
   print PATHOUT "$ENV{HPCALE_REMOTE_JOBDIR}\n";
   print PATHOUT "$ENV{HPCALE_RESERVE_RETRY}\n";
   print PATHOUT "$ENV{HPCALE_SERVER_HOST}\n";
   print PATHOUT "$ENV{HPCALE_SERVER_DB_HOST}\n";
   print PATHOUT "$ENV{HPCALE_SERVER_DB_PORT}\n";
   print PATHOUT "$ENV{HPCALE_SERVER_DB}\n";
   print PATHOUT "$ENV{HPCALE_SERVER_DB_USER}\n";
   print PATHOUT "$ENV{HPCALE_SERVER_DB_PASS}\n";
   close PATHOUT;    
   
   print "\nInstalling Web services ...\n";

   open I, "< ./soapsvc/websvc.lst";  
   my @files = <I>;   
   close I;
   
   &install_websvc(\@files);
   
   `chmod 700 $ENV{APACHE_HOME}/cgi-bin/HPCALE/*.cgi >& /dev/null`;
}

sub install_websvc {
    my $files= shift;

    foreach(@{$files}){
        chomp $_;

        if( $_ =~ m/^#/ ){
            print "  Skipping $_ ...\n";
        }
        elsif( length($_) == 0){
        }
        elsif($_ =~ m/.+\/(.*).pl/){
            open SVCIN, "< soapsvc/$_";
            my @contents = <SVCIN>;
            close SVCIN;
            
            open SVCOUT, "> $ENV{APACHE_HOME}/cgi-bin/HPCALE/$1.cgi";
            print SVCOUT "#!$ENV{HPCALE_PERL_PATH} -w\n";
            foreach(@contents){
                print SVCOUT $_;
            }
            close SVCOUT;

            print "  Installing $1 ...\n";
        }
    }
}

sub install_server {
   print "\nInstalling Server files ...\n";
    
   `cp -R ./server         $ENV{HPCALE_SERVER_HOME}`;
   `cp -R ./util           $ENV{HPCALE_SERVER_HOME}`;
   
   `cp setenv_server.cfg   $ENV{HPCALE_SERVER_HOME}`;
   `cp unsetenv_server.cfg $ENV{HPCALE_SERVER_HOME}`;

}

sub install_ftpdaemon {
   print "\nInstalling FTP daemon ...\n";

   `cp -R ./ftpdaemon $ENV{HPCALE_SERVER_HOME}`;
}



sub check_perl_modules {
    # checking perl modules
    print "\nChecking perl modules ...\n";
    
    unless ($] >= 5.006) {
        die "Sorry, you need at least Perl 5.6\n";
    }

    # Check versions of dependencies.  0 for version = any version acceptible
    # Read the version file
    my $modules = ();
    open(LST, "< $perl_module_lst_file")
            || die("Cannot read \"$perl_module_lst_file\"");
    while(<LST>){
            if (m/^\s*(\S+)\s+(\S+)\s*$/){
               push(@{$modules}, {name=>$1, version=>$2});
            }
    }
    close(LST);
    
    my %missing = ();
    
    # Modules may change $SIG{__DIE__} and $SIG{__WARN__}, so localise them here
    # so that later errors display 'normally'
    foreach my $module (@{$modules}) {
        local $::SIG{__DIE__};
        local $::SIG{__WARN__};
        unless (have_vers($module->{name}, $module->{version})) {
            $missing{$module->{name}} = $module->{version};
            $errors++;
        }
    }
    
    if (%missing) {
        print "\n\n";
        print "HPCALE requires some Perl modules which are either missing from your\n",
        "system, or the version on your system is too old.\n",
        "They can be installed by running (as root) the following:\n";
        foreach my $module (keys %missing) {
            print "   perl -MCPAN -e 'install \"$module\"'\n";
            if ($missing{$module} > 0) {
                print "   Minimum version required: $missing{$module}\n";
            }
        }
        print "\n";
        exit;
    }
    
    print "You have all required Perl modules\n";
    return 1;
}


# vers_cmp is adapted from Sort::Versions 1.3 1996/07/11 13:37:00 kjahds,
# which is not included with Perl by default, hence the need to copy it here.
# Seems silly to require it when this is the only place we need it...
sub vers_cmp {
  if (@_ < 2) { die "not enough parameters for vers_cmp" }
  if (@_ > 2) { die "too many parameters for vers_cmp" }
  my ($a, $b) = @_;
  my (@A) = ($a =~ /(\.|\d+|[^\.\d]+)/g);
  my (@B) = ($b =~ /(\.|\d+|[^\.\d]+)/g);
  my ($A,$B);
  while (@A and @B) {
    $A = shift @A;
    $B = shift @B;
    if ($A eq "." and $B eq ".") {
      next;
    } elsif ( $A eq "." ) {
      return -1;
    } elsif ( $B eq "." ) {
      return 1;
    } elsif ($A =~ /^\d+$/ and $B =~ /^\d+$/) {
      return $A <=> $B if $A <=> $B;
    } else {
      $A = uc $A;
      $B = uc $B;
      return $A cmp $B if $A cmp $B;
    }
  }
  @A <=> @B;
}


# This was originally clipped from the libnet Makefile.PL, adapted here to
# use the above vers_cmp routine for accurate version checking.
sub have_vers {
  my ($pkg, $wanted) = @_;
  my ($msg, $vnum, $vstr);
  no strict 'refs';
  printf("  Checking for %15s %-9s ", $pkg, !$wanted?'(any)':"(v$wanted)");

  eval { my $p; ($p = $pkg . ".pm") =~ s!::!/!g; require $p; };

  $vnum = ${"${pkg}::VERSION"} || ${"${pkg}::Version"} || 0;
  $vnum = -1 if $@;

  if ($vnum eq "-1") { # string compare just in case it's non-numeric
    $vstr = "not found";
  }
  elsif (vers_cmp($vnum,"0") > -1) {
    $vstr = "found v$vnum";
  }
  else {
    $vstr = "found unknown version";
  }

  my $vok = (vers_cmp($vnum,$wanted) > -1);
  print ((($vok) ? "ok: " : " "), "$vstr\n");
  return $vok;
}

