# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package RC_Locker;

BEGIN{
    push (@INC, "$ENV{HPCALE_SERVER_HOME}");
}

use util::Debug;
use util::RemoteDirManager;
use strict;

my $mod = "RC_Locker";
my $pipe= "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";

sub new {
   my $classname  = shift;         # What class are we constructing?
   my $self   = {};                # Allocate new memory

   bless($self, $classname);       # Mark it of the right type

   $self->{LockHeads} = "LockHeads";
   $self->{LockName} = "hpcale_lock";

   return $self;
}

#
# Create hpcale_lock file in each PBS clusterhead.
# For SSH clusters, this is not required.
#
# Requirements.
#   The clustername.pbs.hosts file should contain clusterhead at the first line.
#
sub createRemoteLocks {
   my $self      = shift;
   my $RCFileSet  = shift;
   my $serverdir = shift; # HPCALE Server dir for creating "clusterhead" file.
   my $jobid     = shift; # Need to create lock file to each clusterhead node.

   my $headJobDir = "$ENV{HPCALE_REMOTE_JOBDIR}/$jobid";

   open HEADFILES, "> $serverdir/$self->{LockHeads}";
   foreach(@{$RCFileSet}){
      if($_ =~ /.+\.PBSrc/){
         my $line = `head -1 $serverdir/$_`; $line =~ /(.+),.+/;
         my $headname = $1;

         print HEADFILES "$headname\n";

         my $dirmanager = new RemoteDirManager();
         my $home = $dirmanager->getHomeDir($headname);

         my $lockfile = "$home/$headJobDir/$self->{LockName}";

         `echo "ps" | ssh $headname 'cat - >! $lockfile '`;
      }
   }

   close HEADFILES;
   return 1;
}

#
# remove locks created on each pbs leader
#
sub removeLocks {
    my $self = shift;
    my $serverJobDir = shift;
    my $jobid = shift;

    open HEADFILES, "< $serverJobDir/$jobid/$self->{LockHeads}";
    
    my @heads = <HEADFILES>;
    foreach(@heads){
        chomp $_;
        my $home = (new RemoteDirManager())->getHomeDir($_);

        Debug::logmsg($mod, $pipe, "Remote locks are being removed from $_");
        `ssh $_ 'rm -f $home/$ENV{HPCALE_REMOTE_JOBDIR}/$jobid/$self->{LockName}'`;
    }

    close HEADFILES;
}


1;

