#!/usr/usc/perl -w

# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

use strict;

my $PVM_startnode = $ARGV[0];
my $nNode         = $ARGV[1];
my $PVM_startnode_defined = $ARGV[2];
my $headjobdir    = $ARGV[3];

my @allocnodes = ();
my $added = 0;

open O, "> $headjobdir/pvmadd.log";
print O "nNode: $nNode\n";

#
# Assumption: $PVM_startnode is set before entering checkpvm function
#
sub checkpvm {
    my $pvmaddstr = shift;
    my $cmd = "ssh $PVM_startnode \"$pvmaddstr | grep successful\" | awk '{print \$1}'";

    print O "\n----------entering checkpvm --------\n";
    print O "\nrun command :\n  $cmd\n";

    #
    # iterate until all requested nodes are allocated.
    #
    while($added < $nNode){
        # If we starts pvm first with 8 nodes, 7 nodes are successful
        # and 1 node is added to pvm by default.
        # So, if $PVM_startnode is not defined, we should add 1 to the
        # total number of nodes joined.
        if($PVM_startnode_defined == 0){
            $added = 1;
            $PVM_startnode_defined = 1;
        }

        my $value = `$cmd`;

        if($nNode == 1){
            # In this case, at this time, we just assume that pvm is
            # launched correctly from the node.
            create_pvmstart_nodefile($PVM_startnode);
        }
        else{
            if($value > 0){
                create_pvmstart_nodefile($PVM_startnode);
            }
            else{
                # Case: PVM startup failed
                print O "should wait more..\n";
            }
        }

        chomp $value;
        $added += $value;
        print O "Number of added nodes: <$value>\n" ;
    }

    print O "\nAdding node completed....\n";
    create_pvmstart_nodefile($PVM_startnode);

    print O "----------exit checkpvm --------\n\n";
}

#
# build pvmadd command using pbsnodefile generated by pbs scheduler
#
sub buildpvmadd_cmd(){
    print O "PBSNODEFILE: $ENV{PBS_NODEFILE}\n";
    my (@nodes) = `cat $ENV{PBS_NODEFILE}`;

    my $pvmaddstr = "echo \"add ";

    if($PVM_startnode_defined == 0){
        my $tnode = $nodes[0];
        chomp $tnode;
        $PVM_startnode = $tnode;
        print O "Selected PVM Start node is : $PVM_startnode\n";
    }

    foreach(@nodes){
        chomp $_;
        $pvmaddstr .= "$_ ";
        push @allocnodes, $_;
    }
    $pvmaddstr .= "\" | pvm";

    print O "$pvmaddstr\n";

    return $pvmaddstr;
}

#######################################################
# Start Main Code
#######################################################
my $pvmaddstr = buildpvmadd_cmd();

&build_nodefile();

#
# run pvm with given hosts
#
&checkpvm($pvmaddstr);

#
# wait until lock file removal
#
my $i=0;
while ( -e "$headjobdir/hpcale_lock" ) {
    print O "Waiting...$i\n";
    $i++;
    `sleep 2`;
}

#
# remove pvm daemon remains in tmp directory
#
foreach(@allocnodes){
    print O "Removing pvm temp files\n";
    chomp $_;
    `ssh $_ 'rm /tmp/pvm*'`;
}

close O;
#######################################################
# End Main Code
#######################################################

sub create_pvmstart_nodefile {
    my $node = shift;
    open O3, "> $headjobdir/pvmstartnode";
    print O3 "<$node>";
    close O3;
}

sub build_nodefile {
    open O1, "> $headjobdir/PBS_PVM_Hosts";

    foreach(@allocnodes){
        print O1  "$_\n";
    }

    close O1;
}


#
# This method is somewhat adhoc
# since it just parse the result from qstat. (deprecated)
#
sub buildpvmadd_cmd_from_qstat(){
    my $user = `whoami`;
    my (@arr) = `qstat -n`;

    my $read = 0;
    my $pvmaddstr = "echo \"add ";

    foreach(@arr){
        if($_ =~ /[0-9]+\./){
            $read = 0;
        }

        if($read == 1){
            my @nodes = split /\s+|\+/, $_ ;
            foreach(@nodes){
                if(length($_) > 0){
                    my @details = split /\//, $_;
                    $pvmaddstr .= "$details[0] ";
                    push @allocnodes, $details[0];
                }
            }
        }

        if($_ =~ /iyoon/ ) {
                $read = 1;
        }
    }

    $pvmaddstr .= "\" | pvm";

    return $pvmaddstr;
}

#
# sub create_troyhorse(){
# 
#     `ssh $PVM_startnode \"$pvmaddstr | grep successful | awk '{print \$1}'\"\n`;
# #
# #     open O2, "> $headjobdir/troy$hostname";
# #
# #     print O2 "# PVM Startup Node is : $PVM_startnode\n";
# #     print O2 "ssh $PVM_startnode \"$pvmaddstr | grep successful | awk '{print \$1}'\"\n";
# #
# #     close O2;
# #
# #     `chmod 700 $headjobdir/troy$hostname`;
# }
#
# sub fly_troyhorse(){
#     # actual move pvm startup file at gateway
#     my $actual_fly = "cat troy$hostname | ssh $PVM_startnode \"cat - >! ~/troy$hostname\"";
#     print O "--->$actual_fly\n";
#     `$actual_fly`;
#
#     # add execution permission
#     `ssh $PVM_startnode 'chmod 700 ~/troy$hostname'`;
# }
#
# sub exec_troyhorse(){
#     my $execcmd = "ssh $PVM_startnode '~/troy$hostname'";
#     my $value = `$execcmd`;
#     return $value;
# }

