# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

# This file is to generate '.sql' file for building
# HPCALE_DB automatically.
#


package db;

use strict;

open O, "> create_rc_db.sql";

print O <<HEADER;

#
# \$ID\$
# \$Author\$
# \$Revision\$
#
# Written by Il-Chul Yoon <iyoon\@cs.umd.edu>
#

# Assumption
# Database "HPCALE_DB" is defined as environment variable

-- DROP DATABASE IF EXISTS \${HPCALE_DB};
-- CREATE DATABASE IF NOT EXISTS \${HPCALE_DB};
-- use \${HPCALE_DB};

DROP DATABASE IF EXISTS HPCALE_DB;
CREATE DATABASE IF NOT EXISTS HPCALE_DB;
use HPCALE_DB;

###################################################################################
# available type
#   - cpu
#   - bandwidth for network
CREATE TABLE rctype
(
    type_id     Integer UNSIGNED NOT NULL AUTO_INCREMENT, PRIMARY KEY (type_id),
    type_name   VARCHAR(30)
)
TYPE=INNODB
;

###################################################################################

# example : 1 , redleader.umiacs.umd.edu , xxx.xxx.xxx.xxx
CREATE TABLE rcmgr
(
    mgr_id      Integer UNSIGNED NOT NULL AUTO_INCREMENT, PRIMARY KEY (mgr_id),
    mgr_host    VARCHAR(50) NOT NULL ,
    mgr_hostip  VARCHAR(20)
)
TYPE=INNODB
;

###################################################################################

CREATE TABLE resource
(
    rc_id       Integer UNSIGNED NOT NULL AUTO_INCREMENT, PRIMARY KEY (rc_id),
    mgr_id      Integer UNSIGNED NOT NULL,
    rctype_id   Integer UNSIGNED NOT NULL,
    rcname      VARCHAR(50) NOT NULL,
    
    INDEX mgr_ind (mgr_id),
    INDEX rctype_ind (rctype_id),
    FOREIGN KEY(mgr_id) REFERENCES rcmgr (mgr_id) ON DELETE RESTRICT ON UPDATE CASCADE,
    FOREIGN KEY(rctype_id) REFERENCES rctype (type_id) ON DELETE RESTRICT ON UPDATE CASCADE
)
TYPE=INNODB
;

###################################################################################

CREATE TABLE cluster
(
    cluster_id  Integer UNSIGNED NOT NULL AUTO_INCREMENT, PRIMARY KEY (cluster_id),
    clst_name   VARCHAR(30) NOT NULL ,
    scheduler   VARCHAR(30) NOT NULL ,
    nNode       Integer UNSIGNED NOT NULL,
    available   Integer UNSIGNED NOT NULL,
    rc_id      Integer UNSIGNED NOT NULL,
    
    INDEX rc_ind (rc_id),
    FOREIGN KEY(rc_id) REFERENCES resource (rc_id) ON DELETE RESTRICT ON UPDATE CASCADE
)
TYPE=INNODB
;

###################################################################################

CREATE TABLE node
(
    node_id     Integer UNSIGNED NOT NULL AUTO_INCREMENT, PRIMARY KEY (node_id),
    node_name   VARCHAR(10) NOT NULL ,
    domain      VARCHAR(30) NOT NULL ,
    rc_id      Integer UNSIGNED NOT NULL,
    
    INDEX rc_ind (rc_id),
    FOREIGN KEY(rc_id) REFERENCES resource (rc_id) ON DELETE RESTRICT ON UPDATE CASCADE
)
TYPE=INNODB
;

###################################################################################

CREATE TABLE joblist
(
    job_id      Integer UNSIGNED NOT NULL AUTO_INCREMENT, PRIMARY KEY (job_id),
    clienthost  VARCHAR(30) NOT NULL ,
    clientuser  VARCHAR(30) NOT NULL ,
    allocated   VARCHAR(10) NOT NULL ,
    executed    VARCHAR(10) NOT NULL
)
TYPE=INNODB
;

###################################################################################

CREATE TABLE component
(
    id          Integer UNSIGNED NOT NULL AUTO_INCREMENT, PRIMARY KEY (id),
    compID      VARCHAR(30) NOT NULL ,
    rc_id       Integer UNSIGNED NOT NULL ,
    path        VARCHAR(255) NOT NULL ,
    file        VARCHAR(50) NOT NULL ,
    type        VARCHAR(50) NOT NULL ,
    parallel    Integer UNSIGNED NOT NULL,
    launchprog  VARCHAR(255) NOT NULL ,
    launcharg   VARCHAR(255) NOT NULL ,
    descfile    VARCHAR(255),
    
    INDEX rc_ind (rc_id),
    FOREIGN KEY(rc_id) REFERENCES resource (rc_id) ON DELETE RESTRICT ON UPDATE CASCADE
)
TYPE=INNODB
;

###################################################################################

CREATE TABLE runtimereq
(
    req_id       Integer UNSIGNED NOT NULL AUTO_INCREMENT, PRIMARY KEY (req_id),
    component_id Integer UNSIGNED NOT NULL,
    req_name     VARCHAR(30),
    req_option   VARCHAR(30),

    INDEX comp_ind (component_id),
    FOREIGN KEY(component_id) REFERENCES component (id) ON DELETE RESTRICT ON UPDATE CASCADE
)
TYPE=INNODB
;

###################################################################################

CREATE TABLE dataport
(
    port_id       Integer UNSIGNED NOT NULL AUTO_INCREMENT, PRIMARY KEY (port_id),
    component_id  Integer UNSIGNED NOT NULL,
    port_name     VARCHAR(30) NOT NULL,
    port_type     VARCHAR(30) NOT NULL,
    port_desc     VARCHAR(255) NOT NULL,

    INDEX comp_ind (component_id),
    FOREIGN KEY(component_id) REFERENCES component (id) ON DELETE RESTRICT ON UPDATE CASCADE
)
TYPE=INNODB
;

###################################################################################
INSERT INTO rctype (type_name) VALUES ('cluster');  # such as bugcluster
INSERT INTO rctype (type_name) VALUES ('node');     # such as tau and playclub


HEADER


close O;
