# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package ConnGraph;

BEGIN{
    push (@INC, "$ENV{HPCALE_SERVER_HOME}");
    push (@INC, "$ENV{HPCALE_SERVER_HOME}/server");
}

use util::Debug;
use util::XJDParser;
use db::PerlDB;

use strict;

my $mod = "ConnGraph";
my $pipe = "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";

sub new {
    my $classname  = shift;         # What class are we constructing?
    my $self   = {};                # Allocate new memory

    bless($self, $classname);       # Mark it of the right type

    $self->_init();

    return $self;
}

# "private" method to initialize fields.
sub _init {
    my $self = shift;

    $self->{nodelist} = ();
    $self->{connlist} = ();
}

sub addNode($) {
    my $self = shift;
    my $name = shift;
    my $node = shift;

    $self->{nodelist}->{$name} = $node;
}

sub getNodes {
    my $self = shift;

    return $self->{nodelist};
}

sub addConn($){
    my $self = shift;
    my $name = shift;
    my $conn = shift;

    $self->{connlist}->{$name} = $conn;
}

sub getConns {
    my $self = shift;

    return $self->{connlist};
}

sub toString {
    my $self = shift;
    my $nodelist = $self->{nodelist};
    my $connlist = $self->{connlist};

    my $output = "\n";

    foreach my $key (keys %{$nodelist}){
        $output .= $nodelist->{$key}->toString();
    }

    foreach my $key (keys %{$connlist}){
        $output .= $connlist->{$key}->toString();
    }

    return $output;
}

sub commonInfo {
    my $self = shift;
    my $aComp = shift;
    my $comp  = shift;
    my $rc_id = shift;
    
    my $launchdir = "";
    my $launchprog = "";
    my $launcharg = "";
    
    # get information from database
    ( $aComp->{path},
      $aComp->{file},
      $aComp->{type},
      $aComp->{parallel},
      $launchprog,
      $launcharg) = $self->getCompDetail($aComp->{name}, $rc_id);

    if(! defined $comp->{launchdir}){
        $aComp->{launchdir} = $launchdir;
    }
    else{
        $aComp->{launchdir} = $comp->{launchdir};
    }

    if(! defined $comp->{launchprog}){
        $aComp->{launchprog} = $launchprog;
    }
    else{
        $aComp->{launchprog} = $comp->{launchprog};
    }

    if(! defined $comp->{launcharg}){
        $aComp->{launcharg} = $launcharg;
    }
    else{
        $aComp->{launcharg} = $comp->{launcharg};
    }

    if(! defined $comp->{sendfiles}){
        $aComp->{sendfiles} = "";
    }
    else{
        $aComp->{sendfiles} = $comp->{sendfiles};
    }
    
    if(! defined $comp->{recvfiles}){
        $aComp->{recvfiles} = "";
    }
    else{
        $aComp->{recvfiles} = $comp->{recvfiles};
    }
}


sub genCompInfo_onNode($$){
    my $self = shift;
    my $key = shift;
    my $comp = shift;
    
    my $aComp = ();

    $aComp->{id} = $key;
    $aComp->{name} = $comp->{name};
    $aComp->{node} = $comp->{node};
    $aComp->{domain} = $comp->{domain};
    $aComp->{head} = &getRCManager("node", $comp->{node}, $comp->{domain});

    if(defined $comp->{argument}){
        $aComp->{argument} = $comp->{argument};
    }

    my $rc_id = getNode_RCID ($comp->{node}, $comp->{domain});
    $aComp->{rcid} = $rc_id;
    
    $self->commonInfo($aComp, $comp, $rc_id);
    
    return $aComp;    
}

sub genCompInfo_onCluster($$){
    my $self = shift;
    my $key = shift;
    my $comp = shift;

    my $aComp = ();

    $aComp->{id} = $key;
    $aComp->{name}   = $comp->{name};
    $aComp->{head}      = &getRCManager("cluster", $comp->{cluster});
    $aComp->{nNode}     = $comp->{nNode};
    $aComp->{cluster}   = $comp->{cluster};
    $aComp->{scheduler} = &getScheduler($comp->{cluster});

    if(defined $comp->{argument}){
        $aComp->{argument} = $comp->{argument};
    }

    my $rc_id = getCluster_RCID ($comp->{cluster});
    $aComp->{rcid} = $rc_id;
    
    $aComp->{nNode} = 1 if ($aComp->{nNode} == undef);

    $self->commonInfo($aComp, $comp, $rc_id);

    return $aComp;    
}

sub genCompInfo_onCluster_withNodeSet($$){
    my $self = shift;
    my $key = shift;
    my $comp = shift;
    
    my $aComp = ();
    
    $aComp->{id} = $key;
    $aComp->{name}   = $comp->{name};
    $aComp->{head}      = &getRCManager("cluster", $comp->{cluster});
    $aComp->{nNode}     = $comp->{nNode};
    $aComp->{cluster}   = $comp->{cluster};
    $aComp->{nodeset}   = $comp->{nodeset};
    $aComp->{scheduler} = &getScheduler($comp->{cluster});

    if(defined $comp->{argument}){
        $aComp->{argument} = $comp->{argument};
    }

    my $rc_id = getCluster_RCID ($comp->{cluster});
    $aComp->{rcid} = $rc_id;

    $aComp->{nNode} = 1 if ($aComp->{nNode} == undef);

    $self->commonInfo($aComp, $comp, $rc_id);

    return $aComp;    
}


sub getDetailedEnv {
   my $self = shift;
   my $nodelist = $self->{nodelist};
   my $connlist = $self->{connlist};

   # Compute best candidate for given connection graph
   my $xjd = ();
   $xjd->{ICXJD} = ();
   $xjd->{ICXJD}->{version} = "1.5";
   $xjd->{ICXJD}->{components} = ();
   $xjd->{ICXJD}->{components}->{component} = ();
   $xjd->{ICXJD}->{components}->{component}->{component} = ();

   # write component info   
   foreach my $key (keys %{$nodelist}){
      my $comp = $nodelist->{$key}->getComponent;
      my $aComp = ();
    
      if($nodelist->{$key}->needNode){
         $aComp = $self->genCompInfo_onNode($key, $comp);
      }
      elsif($nodelist->{$key}->needCluster && $nodelist->{$key}->isNodesetDef){
         $aComp = $self->genCompInfo_onCluster_withNodeSet($key, $comp);
      }
      elsif($nodelist->{$key}->needCluster && $nodelist->{$key}->isNodesetDef == 0){
         $aComp = $self->genCompInfo_onCluster($key, $comp);
      }

      # Lastly, assign this component with its name as key
      $xjd->{ICXJD}->{components}->{component}->{component}->{$key} = $aComp;
    }

   # write connection info
   if(keys(%{$connlist}) > 0) {
      $xjd->{ICXJD}->{connections} = ();
      $xjd->{ICXJD}->{connections}->{connection} = ();
      $xjd->{ICXJD}->{connections}->{connection}->{connection} = ();
    
      foreach my $key (keys %{$connlist}){
         my $conn = $connlist->{$key}->getConnection;
    
         $xjd->{ICXJD}->{connections}->{connection}->{connection}->{$key} = $conn;
      }
   }
    
   return $xjd;
}

sub getCompDetail {
    my $self   = shift;
    my $compID = shift;
    my $rc_id   = shift;

    my $perldb = new PerlDB();

    my $query = "select path, file, type, parallel, launchprog, launcharg from component where compID = \"$compID\" and rc_id = $rc_id";

    my $fields = $perldb->multcol_select($query);

    $perldb->db_end ();

    my $path     = $fields->[0]->[0];
    my $file     = $fields->[0]->[1];
    my $type     = $fields->[0]->[2];
    my $parallel = $fields->[0]->[3];
    my $launchprog=$fields->[0]->[4];
    my $launcharg =$fields->[0]->[5];
    
    return ($path, $file, $type, $parallel, $launchprog, $launcharg);
}

sub getRCManager {
    my $rctype = shift;
    
    my $perldb = new PerlDB();
    my $query = "";
    my $rc_id;
    
    my $clustername = "";
    my $nodename = "";
    my $domain = "";

    if($rctype eq "cluster"){
        $clustername = shift;
        
        $query = "select rc_id from cluster where clst_name = \"$clustername\"";
        #print STDERR "$query\n";
        
        $rc_id = $perldb->select($query);
    }
    elsif($rctype eq "node"){
        $nodename = shift;
        $domain = shift;

        $query = "select rc_id from node where node_name = \"$nodename\" and domain = \"$domain\" ";
        #print STDERR "$query\n";

        $rc_id = $perldb->select($query);
    }    
        
    $query = "select mgr_id from resource where rc_id = ".$rc_id->[0];
    #print STDERR "$query\n";

    my $mgr_id = $perldb->select($query);

    $query = "select mgr_host from rcmgr where mgr_id = ".$mgr_id->[0];
    #print STDERR "$query\n";

    my $mgr = $perldb->select($query);

    $perldb->db_end ();

    if($rctype eq "cluster"){
       Debug::dbgmsg(3, $mod, $pipe, "Cluster : $clustername / Manager(Head) : $mgr->[0]");
    }
    elsif($rctype eq "node"){
      Debug::dbgmsg(3, $mod, $pipe, "Node : $nodename / Manager(Head) : $mgr->[0]");
    }

    return $mgr->[0];
}



##############################################################################
# Below, node related information related db operations
##############################################################################
sub getNode_RCID {
    my $nodename = shift;
    my $domain = shift;

    my $perldb = new PerlDB();

    my $query = "select rc_id from node where node_name = \"$nodename\" and domain = \"$domain\" ";
    my $rc_id = $perldb->select($query);
    
    $perldb->db_end ();

    return $rc_id->[0];
}



##############################################################################
# Below, cluster related information related db operations
##############################################################################
sub getScheduler {
    my $cluster = shift;

    my $perldb = new PerlDB();

    my $query = "select scheduler from cluster where clst_name = \"$cluster\"";
    my $scheduler = $perldb->select($query);

    $perldb->db_end ();

    return $scheduler->[0];
}

sub getCluster_RCID {
    my $cluster = shift;

    my $perldb = new PerlDB();

    my $query = "select rc_id from cluster where clst_name = \"$cluster\"";
    my $rc_id = $perldb->select($query);
    
    $perldb->db_end ();

    return $rc_id->[0];
}

1;

