# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package RCDiscover;

BEGIN{
    push (@INC, "$ENV{HPCALE_SERVER_HOME}");
    push (@INC, "$ENV{HPCALE_SERVER_HOME}/server");
}

use util::Debug;
use util::XJDParser;
use discovery::ConnGraph;
use discovery::Node;
use discovery::Conn;
use Data::Dumper;
use strict;

my $mod = "Discovery";
my $pipe= "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";

sub new {
    my $classname  = shift;         # What class are we constructing?
    my $self   = {};                # Allocate new memory

    bless($self, $classname);       # Mark it of the right type

    return $self;
}

sub discoverResource($)
{
    my ($self, $highXJD) = @_;

    my $connGraph = new ConnGraph();

    # Add component
    my $compList = $highXJD->getComponentList();
    
    foreach my $key (keys %{$compList}){
        my $comp = $compList->{$key};

        my $needCluster = &needCluster($comp);
        my $needNode   = &needNode($comp);
        my $nodesetDef = &isNodeSetDefined($comp);

        $connGraph->addNode($key, new Node($comp, $needNode, $needCluster, $nodesetDef));
    }

    # Add connection
    my $connList = $highXJD->getConnectionList();
    foreach my $key (keys %{$connList}){
        my $conn = $connList->{$key};

        my $exporter = $conn->{exporter};
        my $importer = $conn->{importer};
        my $exportport  = $conn->{exportport};
        my $importport  = $conn->{importport};

        $connGraph->addConn($key, new Conn($key, $conn, $exporter, $importer, $exportport, $importport));
    }

    Debug::dbgmsg(3, $mod,$pipe, $connGraph->toString());
     
    my $lowxjd = &computeExecEnv($connGraph);
    Debug::dbgmsg(3, $mod, $pipe, "\n".Dumper($lowxjd)."\n");

    
    return $lowxjd;
}

# Based on connection graph, compute best combination of locations for executing
# the components.
# Now, just return the specified cluster and nodeset which are specified for
# each component.
sub computeExecEnv($){
    my $connGraph = shift;

    return $connGraph->getDetailedEnv ;
}

sub needCluster($) {
    my $comp = shift;

    if(defined ($comp->{cluster})){
        return 1;
    }
    else{
        return 0;
    }
}

sub isNodeSetDefined($) {
    my $comp = shift;

    if(defined($comp->{nodeset})){
        return 1;
    }
    else{
        return 0;
    }
}

sub needNode($) {
    my $comp = shift;

    if(defined ($comp->{node})){
        return 1;
    }
    else{
        return 0;
    }
}


1;

