# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package ComponentExecute;

BEGIN{
    push (@INC, "$ENV{HPCALE_SERVER_HOME}");
    push (@INC, "$ENV{HPCALE_SERVER_HOME}/server");
}

use allocation::RC_Locker;
use execution::JobExecManager;
use util::XJDWriter;
use util::JobRecordManager;
use Data::Dumper;
use strict;

my $mod = "ComponentExecute";
my $pipe= "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";

sub new 
{
   my $classname  = shift;         # What class are we constructing?
   my $jobid = shift;
   
   my $self   = {};                # Allocate new memory

   bless($self, $classname);       # Mark it of the right type

   $self->{jobExManager} = new JobExecManager();
   $self->{jobRecordMgr} = new JobRecordManager();
   $self->{jobid} = $jobid;
   $self->{serverDir} = "$ENV{HPCALE_JOBDIR}";
   
   return $self;
}

sub executeComponent 
{
   my ($self, $lowXJD, $xjdprefix) = @_;

   my $cList = $lowXJD->getComponentList();
   my $jobid = $self->{jobid};
   my $jobdir = "$self->{serverDir}/$self->{jobid}";

   Debug::logmsg($mod, $pipe, "Start component execution...");
   
   # Create XJD file that shall be copied to actual resource to launch component.
   # Remember that one of our target application is the one on top of InterComm v1.5
   # that each application requires XJD file as an input.
   my $xjd = ();
   $xjd->{ICXJD} = $lowXJD->{rc};
   (new XJDWriter())->writeXML($xjd, "$jobdir/$xjdprefix.xjd");

   # launch components and store the information for tracking processes 
   # on remote resources
   my @runningComp = ();    
   foreach my $key (keys %{$cList}){
      my $comp = $cList->{$key};
        
      my $result = $self->{jobExManager}->launch_comp($comp, 
                                                      $key, 
                                                      $jobid, 
                                                      $jobdir, 
                                                      $xjdprefix);
      if($result > 0){
         my $MFile = "MF-".$key."-".$comp->{name}."-".$comp->{rcid};
         push @runningComp, [$MFile, $comp->{file}, $comp->{type}];
      }
      else{
         $self->{jobExManager}->stopExecution (\@runningComp);
         return -1;
      }
   }
 
   $self->{jobRecordMgr}->update_record($jobid, 2, "running");

   # give 3 seconds for the remote resource to start processes
   $self->wait(3);

   # wait the jobs finish
   $self->{jobExManager}->checkJobs($jobdir,
                                    \@runningComp,
                                    $self->{jobRecordMgr},
                                    $jobid
                                   );

   return 1;
}

sub wait 
{
   my ($self, $time) = @_;
   `sleep $time`;
}
  

sub stopExecution 
{
   my ($self, $lowXJD) = @_;
   my $cList = $lowXJD->getComponentList();
   my $jobdir = "$self->{serverDir}/$self->{jobid}";

   # build running component information
   my @runningComp = ();    
   foreach my $key (keys %{$cList}){
      my $comp = $cList->{$key};
        
      my $MFile = "MF-".$key."-".$comp->{name}."-".$comp->{rcid};
      push @runningComp, [$MFile, $comp->{file}, $comp->{type}];
   }
  
   $self->{jobExManager}->stopExecution($jobdir,
                                        \@runningComp,
                                        $self->{jobRecordMgr},
                                        $self->{jobid}
                                       );
   return 1;
}


1;
