# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package ExecuteSEQComp;

BEGIN{
    push (@INC, "$ENV{HPCALE_SERVER_HOME}/server");
}

use util::Debug;
use util::RemoteDirManager;
use strict;

my $mod = "ExecuteSEQComp";
my $pipe = "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";

sub new {
    my $classname  = shift;         # What class are we constructing?
    my $self   = {};                # Allocate new memory

    bless($self, $classname);       # Mark it of the right type

    return $self;
}

sub executeComponent {
   my ($self, $comp, $key, $jobid, $jobdir, $xjdprefix) = @_;

   my $path = $comp->{path};
   my $file = $comp->{file};
   my $head = $comp->{head};
   my $MFile = "MF-".$key."-".$comp->{name}."-".$comp->{rcid};

   if(defined $comp->{cluster}){
      if(defined $comp->{nodeset}){
         $head =~ /[a-zA-Z0-9]+\.(.*)/;       
         my $domain = $1;

         open MF, "> $jobdir/$MFile";
         my @nodes = split /,/ , $comp->{nodeset} ;        

         
         foreach(@nodes){
            chomp $_;
            print MF "$_.$domain\n";
         } 
         close MF;
      }
      else{
         $self->genClusterMachineFile($jobdir, $MFile, $comp);          
      }                        
   }
   else{
      open MF, "> $jobdir/$MFile" or return -1;
      print MF "$comp->{node}.$comp->{domain}\n";
      close MF;
   }

   # obtain launch node 
   my $launchnode = `head -1 $jobdir/$MFile`; chomp $launchnode;
   
   # set remote home and job directory
   my $rhome = (new RemoteDirManager())->getHomeDir($launchnode);
   my $rdir  = "$ENV{HPCALE_REMOTE_JOBDIR}/$jobid";

   # write machine file and XJD file on remote resource 
   `cat $jobdir/$MFile | ssh $head 'cat - >! $rhome/$rdir/$MFile '`;
   `cat $jobdir/$xjdprefix.xjd | ssh $head 'cat - >! $rhome/$rdir/$xjdprefix.xjd '`;

   # generate command to launch component
   Debug::logmsg($mod, $pipe, "Generating execution command");

   my $prefix = "ssh $launchnode '/bin/sh -c \" "; 
   my $postfix= " \"' ";

   my $cd = "";
   if(length($comp->{launchdir}) > 0){
      $cd = "cd " . $comp->{launchdir};
   }
   else{
      $cd = "cd $rhome/$rdir"; 
   }

   # set program path, name and arguments 
   my $prog = "$path/$file"; # set program path and name
   my $progarg = "";
   if(defined $comp->{argument}){
      $progarg = " $comp->{argument} ";
      $progarg =~ s/__MFILE__/$rhome\/$rdir\/$MFile/g;
      $progarg =~ s/__RHOME__/$rhome\/$rdir/g;
   }

   # set xjd arguments
   my $xjd = "$rhome/$rdir/$xjdprefix.xjd";

   # set program output pipes
   my $stdout = "$rhome/$rdir/$key-STDOUT.log";
   my $stderr = "$rhome/$rdir/$key-STDERR.log";
   
   Debug::dbgmsg(3, $mod, $pipe, "[$prefix]");
   Debug::dbgmsg(3, $mod, $pipe, "[$cd]");
   Debug::dbgmsg(3, $mod, $pipe, "[$prog]");
   Debug::dbgmsg(3, $mod, $pipe, "[$progarg]");
   Debug::dbgmsg(3, $mod, $pipe, "[$stdout]");

   my $command = "$prefix $cd; $prog $progarg >$stdout 2>$stderr $postfix";

   Debug::logmsg($mod, $pipe, "[$command]");

   # background execution and return
   my $v = system "$command &";
   ($v == 0) ? return 1 : return -1; 
}

sub genClusterMachineFile {
    my ($self, $jobdir, $MFile, $comp) = @_;
    
    my $head = $comp->{head};
    $head =~ /[a-zA-Z0-9]+\.(.*)/;       
    my $domain = $1;
    
    if($comp->{scheduler} eq "PBS"){
        open PBSNODE, "< $jobdir/PBSNODES-$head";
    }
    
    my @nodes = <PBSNODE>;
    close PBSNODE;
    
    open MF, "> $jobdir/$MFile";
    open TMP, "> $jobdir/tmp";

    my $i = 0;            
    foreach(@nodes){
        chomp $_;
        if($i < $comp->{nNode}){
            print MF "$_.$domain\n";
            $i++;
        }
        else{
            print TMP "$_\n";
        }
    }
    close MF;
    close TMP;
}


1;
