# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package DiscoveryNotifier;

BEGIN{
    push (@INC, "$ENV{HPCALE_SERVER_HOME}/server");
}

use util::Debug;
use notification::Notifier;
use Data::Dumper;
use strict;

my $mod = "DiscoveryNotifier";
my $pipe= "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";

our @ISA = qw(Notifier);

sub new 
{
   my $classname  = shift;         # What class are we constructing?
   my ($clienturi, $clientproxy) = @_;
    
   my $self = Notifier->new(@_);

   $self->{module} = "Discovery";
   Debug::logmsg($mod, $pipe, "$mod is registered");
    
   return bless($self, $classname);
}

sub notify 
{
   my ($self, $lowXJD) = @_;
   my $clientproxy = $self->{clientproxy};

   Debug::logmsg($mod, $pipe, "Notifying to $clientproxy");

   # actual send the refined low-level XJD to the client
   my $result = $self->{soap}->discoveryResult($lowXJD);

   unless($result->fault){
      Debug::logmsg($mod, $pipe, "clientd returns normally");
      return 1;
   }
   else{
      my $err = join ', ', $result->faultcode, 
                           $result->faultstring, 
                           $result->faultdetail;
      Debug::errmsg($mod, $pipe, "Webservice fault : $err");
      return -1;            
   }
}

1;
