# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package ExecutionNotifier;

BEGIN{
    push (@INC, "$ENV{HPCALE_SERVER_HOME}/server");
}

use util::Debug;
use notification::Notifier;
use Data::Dumper;
use strict;

my $mod = "ExecutionNotifier";
my $pipe= "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";

our @ISA = qw(Notifier);

sub new 
{
   my $classname  = shift;         # What class are we constructing?
   my ($clienturi, $clientproxy) = @_;
   
   my $self = Notifier->new(@_);

   $self->{module} = "Execution";
   Debug::logmsg($mod, $pipe, "$mod is registered.");

   return bless($self, $classname);
}

#
# to send back the output files to client
#
sub ready2send
{
   my ($self,$jobid,$head,$specfiles,$stdout,$stderr) = @_;
   my $clientproxy = $self->{clientproxy};

   Debug::logmsg($mod, $pipe, "Ask clientd to receive files (from $head).");

   # ask clientd to receive output files
   # wait ...
   my $result = $self->{soap}->txfer_outfiles($jobid,$head,$specfiles,
                                              $stdout,$stderr);

   unless($result->fault){
      my $r = $result->result();
      if($r == 1){
         Debug::logmsg($mod, $pipe, "clientd received output files.");
         return 1;
      }
      else{
         Debug::errmsg($mod, $pipe, "clientd failed to receive output files.");
         return -1;
      }
   }
   else{
      my $err = join ', ', $result->faultcode, 
                           $result->faultstring, 
                           $result->faultdetail;
      Debug::errmsg($mod, $pipe, "Webservice fault : $err");
      return -1;            
   }
}

sub notify
{
   my ($self, $flag) = @_;
   my $clientproxy = $self->{clientproxy};

   Debug::logmsg($mod, $pipe, "Notifying to $clientproxy : $flag");

   my $result = $self->{soap}->executionResult($flag);

   unless($result->fault){
      Debug::logmsg($mod, $pipe, "clientd returns normally");
      return 1;
   }
   else{
      my $err = join ', ', $result->faultcode,
                           $result->faultstring,
                           $result->faultdetail;
      Debug::errmsg($mod, $pipe, "Webservice fault : $err");
      return -1;
   }
}

1;
