#
# Written by Il-Chul Yoon <iyoon@cs.umd.edu>
#

package Notifier;

BEGIN{
    push (@INC, "$ENV{HPCALE_SERVER_HOME}");
}

use util::Debug;
use Data::Dumper;
use SOAP::Lite;
use strict;

my $mod = "Notifier";
my $pipe= "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";

sub new {
   my $classname = shift;
   my ($clienturi, $clientproxy) = @_;
    
   my $self = {};
   $self->{module} = undef;
   $self->{clienturi} = $clienturi;
   $self->{clientproxy} = $clientproxy;

   Debug::logmsg($mod, $pipe, "Client Info: URI: $clienturi, PROXY: $clientproxy");

   $self->{soap} = new SOAP::Lite(uri => $self->{clienturi}, proxy => $self->{clientproxy});

   bless($self, $classname);    
}

sub forkFailed {
   my $self = shift;
  
   my $str = "forkFailed is invoked from Server module(".$self->{module}.")";
   Debug::errmsg(1, $mod, $pipe, $str); 
    
   my $result = $self->{soap}->errmsg($self->{module}, "Can't start service");

   return $self->errorcheck($result);        
}    

sub forkOk {
   my $self = shift;
   my $pid = shift;
   my $result = "";
   
   unless ($pid == 0){
      Debug::dbgmsg(2, $mod, $pipe, "Parent PID is $pid.");
      $result = $self->{soap}->logmsg($self->{module}, "Parent PID is $pid.");
   }
   else{
      Debug::dbgmsg(2, $mod, $pipe, "Child PID(0) is ongoing...");
      $result = $self->{soap}->logmsg($self->{module}, "Child PID(0) is ongoing.");
   }

   return $self->errorcheck($result);        
}


sub errorcheck {
   my $self = shift;
   my $result = shift;  
    
   unless ($result->fault){
      if($result -> result() < 0){
         Debug::errmsg($mod, $pipe, "Client daemon returns error.");
         return -1;
      }
   }
   else{
      my $err = join ', ', $result->faultcode, 
                           $result->faultstring, 
                           $result->faultdetail;
      Debug::logmsg($mod, $pipe, "Webservice fault : $err");
      return -1;            
   }

   return 1;
}      
    
1;
