# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package RTEnvNotifier;

BEGIN{
    push (@INC, "$ENV{HPCALE_SERVER_HOME}/server");
}

use util::Debug;
use notification::Notifier;
use Data::Dumper;
use strict;

my $mod = "RTEnvNotifier";
my $pipe= "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";

our @ISA = qw(Notifier);

sub new 
{
   my $classname  = shift;         # What class are we constructing?
   my ($clienturi, $clientproxy) = @_;
   
   my $self = Notifier->new(@_);

   $self->{module} = "RTEnvSetup";
   Debug::logmsg($mod, $pipe, "$mod is registered.");
   
   return bless($self, $classname);
}

sub ready2recv
{
   my ($self,$jobid,$specfiles) = @_;
   my $clientproxy = $self->{clientproxy};

   Debug::logmsg($mod, $pipe, "Ask clientd to transfer files.");

   # ask clientd to transfer input files
   # wait ...
   my $result = $self->{soap}->txfer_infiles($jobid,$specfiles);

   unless($result->fault){
      my $r = $result->result();
      if($r == 1){
         Debug::logmsg($mod, $pipe, "clientd transferred input files.");
         return 1;
      }
      else{
         Debug::errmsg($mod, $pipe, "clientd failed to transfer input files.");
         return -1;
      }
   }
   else{
      my $err = join ', ', $result->faultcode, 
                           $result->faultstring, 
                           $result->faultdetail;
      Debug::errmsg($mod, $pipe, "Webservice fault : $err");
      return -1;            
   }
}

sub notify 
{
   my ($self, $flag) = @_;
   my $clientproxy = $self->{clientproxy};

   Debug::dbgmsg(1, $mod, $pipe, "Notifying to $clientproxy : $flag");
        
   my $result = $self->{soap}->setupRTEnvResult($flag);
  
   unless($result->fault){
      Debug::logmsg($mod, $pipe, "clientd returns normally");
      return 1;
   }
   else{
      my $err = join ', ', $result->faultcode, 
                           $result->faultstring, 
                           $result->faultdetail;
      Debug::errmsg($mod, $pipe, "Webservice fault : $err");
      return -1;            
   }
}

1;
