# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package FileTransferHandler;

BEGIN{
    push (@INC, "$ENV{HPCALE_SERVER_HOME}");
}

use util::Debug;
use util::RemoteDirManager;
use Data::Dumper;
use strict;

my $mod = "FileTransferHandler";
my $pipe = "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";


sub new {
    my $classname  = shift;         # What class are we constructing?
    my $jobid = shift;
    my $jobdir = shift;
    my $chost = shift;
    my $user = shift;
        
    my $self   = {};                # Allocate new memory
    bless($self, $classname);       # Mark it of the right type

    $self->{name} = "FTP";
    $self->{Members} = ();
    $self->{jobid} = $jobid;
    $self->{jobdir} = $jobdir;
    $self->{chost} = $chost;
    $self->{user} = $user;
    
    Debug::logmsg($mod, $pipe, "FileTransfer Handler is registered");
    return $self;
}

sub reorganize
{
   my ($self,$comp,$key) = @_;
   return $self->prepare($comp,$key);
}

# 
# if this function is invoked, it means given component has 
# sendfiles or recvfiles
#
sub prepare {
   my ($self,$comp,$key) = @_;
    
   # $key means the unique id of each component in a XJD file
   $self->{Members}->{$key} = ();
   $self->{Members}->{$key}->{comp} = $comp;
   $self->{Members}->{$key}->{sendfiles} = undef;
   $self->{Members}->{$key}->{recvfiles} = undef;
   $self->{Members}->{$key}->{stdout} = undef;
   $self->{Members}->{$key}->{stderr} = undef;
   #$self->{Members}->{$key}->{hpcale_log} = undef;
    
   Debug::dbgmsg(3, $mod, $pipe, "[FTP] COMP : $comp->{name}, KEY : $key");
    
   if(length($comp->{sendfiles}) > 0){
      Debug::logmsg($mod, $pipe, "Registering to FTP handler: sendfiles '$comp->{sendfiles}' for $comp->{name} ($key)");
      $self->{Members}->{$key}->{sendfiles} = $comp->{sendfiles};
   }
   if(length($comp->{recvfiles}) > 0){
      Debug::logmsg($mod, $pipe, "Registering to FTP handler: recvfiles '$comp->{recvfiles}' for $comp->{name} ($key)");
      $self->{Members}->{$key}->{recvfiles} = $comp->{recvfiles};
    }

   $self->{Members}->{$key}->{stdout} = "$key-STDOUT.log";
   $self->{Members}->{$key}->{stderr} = "$key-STDERR.log";

   # when we do this, it means to copy logfile from the resource. the log 
   # is located at the frontend, not resource. So, we cannot do this.
   # if client want to receive the log, he can specify the logfile in the 
   # recvfiles tag in XJD
   #$self->{Members}->{$key}->{hpcale_log} = $pipe;

   return 1;        
}

#
# let the client daemon transfer input files thru ftpd.
# 
sub beReady {
   my ($self, $notifier) = @_;
   my $result = 1;
   foreach my $key (keys %{$self->{Members}}){
      Debug::dbgmsg(3, $mod, $pipe, "Key is :$key"); 
      Debug::dbgmsg(3, $mod, $pipe, "JobID is :$self->{jobid}"); 
      Debug::dbgmsg(3, $mod, $pipe, "Head is :$self->{Members}->{$key}->{comp}->{head}"); 
      Debug::dbgmsg(3, $mod, $pipe, "Sendfiles: $self->{Members}->{$key}->{sendfiles}");
      Debug::dbgmsg(3, $mod, $pipe, "Recvfiles: $self->{Members}->{$key}->{recvfiles}");
      Debug::dbgmsg(3, $mod, $pipe, "stdout: $self->{Members}->{$key}->{stdout}");
      Debug::dbgmsg(3, $mod, $pipe, "stderr: $self->{Members}->{$key}->{stderr}");
      #Debug::dbgmsg(3, $mod, $pipe, "stderr: $self->{Members}->{$key}->{hpcale_log}");

      $result = $notifier->ready2recv($self->{jobid},
                                      $self->{Members}->{$key}->{sendfiles});

      if($result == -1){
         # if fails during this foreach loop,
         # the files already transferred may stay in the remote machines.
         Debug::errmsg($mod, $pipe, "input file transfer from component $key: failed");
         return -1;
      }
      else{
         Debug::logmsg($mod, $pipe, "input file transfer from component $key: ok");
      }
   }

   return 1;
}

#
# let the client takes the output files thru ftpd.
#
sub back_transfer {
   my ($self, $notifier) = @_; # Here, notifier is ExecutionNotifier.
   Debug::logmsg($mod, $pipe, "Output files are ready to be taken");

   my $result = 1;
   foreach my $key (keys %{$self->{Members}}){
      Debug::dbgmsg(3, $mod, $pipe, "Key is :$key"); 
      Debug::dbgmsg(3, $mod, $pipe, "JobID is :$self->{jobid}"); 
      Debug::dbgmsg(3, $mod, $pipe, "Head is :$self->{Members}->{$key}->{comp}->{head}"); 
      Debug::dbgmsg(3, $mod, $pipe, "Sendfiles: $self->{Members}->{$key}->{sendfiles}");
      Debug::dbgmsg(3, $mod, $pipe, "Recvfiles: $self->{Members}->{$key}->{recvfiles}");
      Debug::dbgmsg(3, $mod, $pipe, "stdout: $self->{Members}->{$key}->{stdout}");
      Debug::dbgmsg(3, $mod, $pipe, "stderr: $self->{Members}->{$key}->{stderr}");
      #Debug::dbgmsg(3, $mod, $pipe, "hpcalelog: $self->{Members}->{$key}->{hpcale_log}");
   
      $result = $notifier->ready2send($self->{jobid},
                                      $self->{Members}->{$key}->{comp}->{head},
                                      $self->{Members}->{$key}->{recvfiles},
                                      $self->{Members}->{$key}->{stdout},
                                      $self->{Members}->{$key}->{stderr});

      if($result == -1){
         # if fails during this foreach loop,
         # the files already transferred may stay in the client machines.
         Debug::errmsg($mod, $pipe, "output file transfer from component $key: failed");
         return -1;
      }
      else{
         Debug::logmsg($mod, $pipe, "output file transfer from component $key: ok");
      }
   }

   return 1;
}




sub cleanup {
   my $self = shift;

   Debug::logmsg($mod, $pipe, "=======================================");
   Debug::logmsg($mod, $pipe, "FTP Handler Cleanup... ");
   Debug::logmsg($mod, $pipe, "=======================================");

   #
   # What to do for clean? 
   #
   # code here...
}

sub error {
   my $self = shift;
   my $name = shift;
   my $head = shift;
    
   Debug::logmsg($mod, $pipe, "File Transfer failed");
}
    
1;
