# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package PVMHandler;

BEGIN{
    push (@INC, "$ENV{HPCALE_SERVER_HOME}");
}

use util::Debug;
use util::RemoteDirManager;
use Data::Dumper;
use strict;

my $mod = "PVMHandler";
my $pipe = "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";

sub new {
   my $classname  = shift;         # What class are we constructing?
   my $jobid = shift;
   my $serverDir = shift;
        
   my $self   = {};                # Allocate new memory
   bless($self, $classname);       # Mark it of the right type
  
   $self->{name} = "PVM";
   $self->{jobid} = $jobid;
   $self->{serverDir} = $serverDir;
   $self->{PVMStartNode} = undef;
   $self->{Member} = ();
   $self->{PVMWaitRetry} = 7;
   
   Debug::logmsg($mod, $pipe, "PVM Handler is registered");
   return $self;
}

sub reorganize 
{
   my ($self,$comp,$key) = @_;
   
   my $head = $comp->{head};
   $head =~ /[a-zA-Z0-9]+\.(.*)/;       
   my $domain = $1;

   my $MFile = "MF-".$key."-".$comp->{name}."-".$comp->{rcid};
   open MF, "< $self->{serverDir}/$MFile" or return -1;
   my @m = <MF>;
   close MF;

   foreach(@m){
      chomp $_;
      Debug::dbgmsg(3, $mod, $pipe, "reorganized: $_");
      push @{$self->{Member}}, "$_.$domain" ;
   }

   if(not defined $self->{PVMStartNode}){
      open PVMSNODE, "< $self->{serverDir}/PVMStartNode";
      my @str = <PVMSNODE>;
      close PVMSNODE;
   
      chomp $str[0];
      Debug::dbgmsg(3, $mod, $pipe, "PVM Start node reorganized: $str[0]");
      $self->{PVMStartNode} = $str[0];
   }

   return 1;
}

sub prepare {
   my ($self,$comp,$key) = @_;
    
   my $head = $comp->{head};
   $head =~ /[a-zA-Z0-9]+\.(.*)/;       
   my $domain = $1;
    
   open PVMNODES, ">> $self->{serverDir}/PVMNodes" or return -1;

   # one MFile for each component in XJD
   my $MFile = "MF-".$key."-".$comp->{name}."-".$comp->{rcid};
   Debug::dbgmsg(3, $mod, $pipe, "Machinefile: $MFile"." for ".$comp->{name});
    
   if(defined $comp->{node}){
      print PVMNODES "$comp->{node}.$comp->{domain}\n";

      open MF, "> $self->{serverDir}/$MFile";
      print MF "$comp->{node}.$comp->{domain}\n";
      close MF; 
   }
   elsif(defined $comp->{cluster}){
      if(defined $comp->{nodeset}){
         open MF, "> $self->{serverDir}/$MFile";
         my @nodes = split /,/ , $comp->{nodeset} ;        
         foreach(@nodes){
            chomp $_;
            if($self->isExist($_, $domain) == 0){
               push @{$self->{Member}}, "$_.$domain" ;
               print PVMNODES "$_.$domain\n";
               print MF "$_.$domain\n";
            }
         } 
         close MF;
      }
      else{
         my @nodes = ();
         if($comp->{scheduler} eq "PBS"){
            open PBSNODE, "< $self->{serverDir}/PBSNODES-$head";
            @nodes = <PBSNODE>;
            close PBSNODE;
         }
            
         open MF, "> $self->{serverDir}/$MFile";
         open TMP, "> $self->{serverDir}/tmp";
         my $i = 0;            
         foreach(@nodes){
            chomp $_;
                
            if($i < $comp->{nNode}){
               push @{$self->{Member}}, "$_.$domain" ;
               print PVMNODES "$_.$domain\n";
               print MF "$_.$domain\n";
               $i++;
            }
            else{
               print TMP "$_\n";
            }
         }
         close MF;
         close TMP;
           
         # At last, PBSNODES-$head file will become size 0 after 
         # being divided into machine files for components
         `mv $self->{serverDir}/tmp $self->{serverDir}/PBSNODES-$head`;
      }
   }
  
   my $mfile_contents = `cat $self->{serverDir}/$MFile`;
   Debug::dbgmsg(3, $mod, $pipe, "Machine file contents:\n$mfile_contents");
    
   close PVMNODES;

   return 1;
}

sub beReady {
   my $self = shift;
   my $addstr = "echo \"add ";
   my $start = 1;
   my $totalNum = scalar @{$self->{Member}};
   
   foreach(@{$self->{Member}}){
      # clear /tmp directory for correct PVM initialization
      Debug::dbgmsg(3, $mod, $pipe, "remove PVM temp files at $_");
      `ssh $_ '/bin/sh -c \"rm /tmp/pvm* 2> /dev/null\"'`;

      # decide PVM start node
      if($start == 1){
         $self->{PVMStartNode} = $_;
         $start = 0;
      }
      else{
         $addstr .= "$_ ";
      }
   }

   $addstr .= "\" | pvm";
   if(not defined $self->{PVMStartNode}){
      return -1;
   }
   else{
      open PVMSNODE, "> $self->{serverDir}/PVMStartNode";
      print PVMSNODE $self->{PVMStartNode};
      close PVMSNODE;
   }

   Debug::dbgmsg(3, $mod, $pipe, "Add str : $addstr");
    
   # launch PVM with the required nodes.
   `ssh $self->{PVMStartNode} '$addstr'`;

   if( $self->waitPVMJoin($totalNum) > 0){
      Debug::logmsg($mod, $pipe, "All nodes are allocated.");
      return 1;
   }
   else{
      Debug::logmsg($mod, $pipe, "PVM join failed. Must be cleaned up");
      return -1;
   }
}

# check the number of allocated hosts
# and compare it to the number of hosts asked
# every 3 seconds.
sub waitPVMJoin {
   my ($self, $numHosts) = @_;

   my $i = 1;
   while($self->checkReservation($numHosts, $i) < 0){
      return -1 if($i >= $self->{PVMWaitRetry});
      $i++;

      system("sleep 3");
   }

   return 1;
}

sub checkReservation {
   my ($self, $numHosts, $i) = @_;

   my $ask_reservation = " ssh $self->{PVMStartNode} \"echo 'conf' | pvm\" | grep 'hosts,' | awk '{print \$1}'";
   my $allocated = `$ask_reservation`;

   if($numHosts == $allocated){
      return 1;
   }
   else{
      Debug::dbgmsg(3, $mod, $pipe, "Asked: ".$numHosts.", Allocated: $allocated");
      Debug::dbgmsg(3, $mod, $pipe, `ssh $self->{PVMStartNode} \"echo 'conf' | pvm\"`);
      return -1;
    }
}
    
sub cleanup {
   my $self = shift;

   Debug::logmsg($mod, $pipe, "=======================================");
   Debug::logmsg($mod, $pipe, "PVM Handler Cleanup : Killing PVM at $self->{PVMStartNode}");
   Debug::logmsg($mod, $pipe, "=======================================");

   # Kill PVM
   `ssh $self->{PVMStartNode} 'echo \"halt\" | pvm > /dev/null'`;   
}

sub error {
   my $self = shift;
   my $name = shift;
   my $head = shift;
    
   Debug::logmsg($mod, $pipe, "PVM management for $name on $head failed");
}
    
sub isExist {
   my $self = shift;
   my $key = shift;
   my $domain = shift;
    
   foreach(@{$self->{Member}}){
      return 1 if($_ eq "$key.$domain");
   }

   return 0;
}

1;
