# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

BEGIN{
   open (I, ".hpcale-libpath");
   my @libs = <I>;
   close(I);

   chomp($libs[0]);
   $ENV{HPCALE_SERVER_HOME} = $libs[0];

   chomp($libs[1]);
   my @pmarray = split /:/ , $libs[1];

   for(2..13){
      chomp($libs[$_]);
   }

   $ENV{HPCALE_LOGPATH} = $libs[2];
   $ENV{HPCALE_LOGFILE} = $libs[3];
   $ENV{HPCALE_DEBUG_LEVEL} = $libs[4];
   $ENV{HPCALE_JOBDIR} = $libs[5];
   $ENV{HPCALE_REMOTE_JOBDIR} = $libs[6];
   $ENV{HPCALE_RESERVE_RETRY} = $libs[7];
   $ENV{HPCALE_SERVER_HOST} = $libs[8];
   $ENV{HPCALE_SERVER_DB_HOST} = $libs[9];
   $ENV{HPCALE_SERVER_DB_PORT} = $libs[10];
   $ENV{HPCALE_SERVER_DB} = $libs[11];
   $ENV{HPCALE_SERVER_DB_USER} = $libs[12];
   $ENV{HPCALE_SERVER_DB_PASS} = $libs[13];
   
   foreach(@pmarray){
      push (@INC, $_);
   }
   push (@INC, "$ENV{HPCALE_SERVER_HOME}");
   push (@INC, "$ENV{HPCALE_SERVER_HOME}/server");
}

use SOAP::Transport::HTTP;

SOAP::Transport::HTTP::CGI
    -> dispatch_to('soapsvc')
    -> handle;


package soapsvc;

use util::Debug;
use allocation::RCAllocate;
use notification::AllocationNotifier;
use strict;

my $mod = undef;
my $pipe= undef;

sub allocateResource {
   my ($self, $user, $host, $lowXJD, $clienturi, $clientproxy) = @_;

   $mod = "AllocationService(CGI)";
   $pipe= "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";
   
   Debug::dbgmsg(1, $mod, $pipe, "========= ALLOCATION =========");
   Debug::dbgmsg(2, $mod, $pipe, "CLIENT URI: $clienturi");
   Debug::dbgmsg(2, $mod, $pipe, "CLIENT PROXY: $clientproxy");

   my $notifier = new AllocationNotifier($clienturi, $clientproxy);
    
   my $child_pid;        
   if (!defined($child_pid = fork())) {
      Debug::logmsg($mod, $pipe, "Can't fork");
      $notifier->forkFailed ;
      return -1;
   } 
   # this is parent (pid > 0)
   elsif ($child_pid) {
      # just let the client know fork is ok. Refer svc_discover_rc.pl
      $notifier->forkOk($child_pid);
      return 1;
   } 
   # this is child (pid 0)
   # for more info, refer svc_discover_rc.pl
   else {
      if($notifier->forkOk (0) < 0){
         Debug::logmsg($mod, $pipe, "Communication Error with client daemon");
      }
      else{
         my ($jobid, $flag) = (new RCAllocate())->allocateResource($user,
                                                                   $host,
                                                                   $lowXJD);
         if($flag > 0){
            Debug::logmsg($mod, $pipe, "Reservation: ok");
            $self->notify_result($notifier, $jobid, 1);
         }
         else{
            Debug::logmsg($mod, $pipe, "Reservation: failed");
            $self->notify_result($notifier, $jobid, $flag);
         }
      }

      return; # just to finish here
   }

   Debug::logmsg($mod, $pipe, "Unknown Error");
   return -1;
}

sub notify_result 
{
   my ($self, $notifier, $jobid, $flag) = @_;
   
   if($notifier->notify($jobid, $flag) == 1){
      Debug::logmsg($mod, $pipe, "client notification: ok");
   }
   else{
      Debug::logmsg($mod, $pipe, "client notification: failed");
   }
}

