# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

BEGIN{
   open (I, ".hpcale-libpath");
   my @libs = <I>;
   close(I);

   chomp($libs[0]);
   $ENV{HPCALE_SERVER_HOME} = $libs[0];

   chomp($libs[1]);
   my @pmarray = split /:/ , $libs[1];

   for(2..13){
      chomp($libs[$_]);
   }

   $ENV{HPCALE_LOGPATH} = $libs[2];
   $ENV{HPCALE_LOGFILE} = $libs[3];
   $ENV{HPCALE_DEBUG_LEVEL} = $libs[4];
   $ENV{HPCALE_SERVER_HOST} = $libs[8];
   $ENV{HPCALE_SERVER_DB_HOST} = $libs[9];
   $ENV{HPCALE_SERVER_DB_PORT} = $libs[10];
   $ENV{HPCALE_SERVER_DB} = $libs[11];
   $ENV{HPCALE_SERVER_DB_USER} = $libs[12];
   $ENV{HPCALE_SERVER_DB_PASS} = $libs[13];

   foreach(@pmarray){
       push (@INC, $_);
   }
   push (@INC, "$ENV{HPCALE_SERVER_HOME}");
   push (@INC, "$ENV{HPCALE_SERVER_HOME}/server");
}

use SOAP::Transport::HTTP;

SOAP::Transport::HTTP::CGI
    -> dispatch_to('soapsvc')
    -> handle;


package soapsvc;

use util::Debug;
use discovery::RCDiscover;
use notification::DiscoveryNotifier;
use strict;

my $mod = undef;
my $pipe= undef;

sub discoverResource {
   my ($self, $highXJD, $clienturi, $clientproxy) = @_;

   $mod = "DiscoveryService(CGI)";
   $pipe= "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";

   # write on new log file
   unlink $pipe if(-e $pipe);

   Debug::dbgmsg(1, $mod, $pipe, "========= DISCOVERY =========");
   Debug::dbgmsg(2, $mod, $pipe, "CLIENT URI: $clienturi");
   Debug::dbgmsg(2, $mod, $pipe, "CLIENT PROXY: $clientproxy");
    
   my $notifier = new DiscoveryNotifier($clienturi, $clientproxy);

   my $child_pid;        
   if (!defined($child_pid = fork())){
      Debug::logmsg($mod, $pipe, "Can't fork");
      $notifier->forkFailed ;
      return -1;
   }
   # this is parent (pid > 0)
   elsif ($child_pid){
      # just let the client know fork is ok.
      # frankly, parent does not know whether child process finished after 
      # handling everything well. So, it has no other option but to just return 
      $notifier->forkOk ($child_pid);
      return 1;
   } 
   # this is child (pid 0)
   #  - although child returns, there is no receiver since parent may already 
   #    return. So, the return value here is actually not meaningful to the client.
   else{
      if($notifier->forkOk (0) < 0){
         Debug::logmsg($mod, $pipe, "Communication Error with client daemon");
      }
      else{
         my $lowXJD = (new RCDiscover())->discoverResource($highXJD);

         if(defined $lowXJD){
            Debug::logmsg($mod, $pipe, "XJD refinement: ok");
         }
         else{
            Debug::logmsg($mod, $pipe, "XJD refinement: failed");
         }

         $self->notify_result($notifier, $lowXJD);
      }

      return; # just to finish here
   } 

   Debug::logmsg($mod, $pipe, "Unknown Error");
   return -1;
}

sub notify_result
{
   my ($self, $notifier, $lowXJD) = @_;

   if($notifier->notify($lowXJD) == 1){
      Debug::logmsg($mod, $pipe, "client notification ok");
   }
   else{
      Debug::logmsg($mod, $pipe, "client notification failure");
   }
}

