# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

BEGIN{
   open (I, ".hpcale-libpath");
   my @libs = <I>;
   close(I);

   chomp($libs[0]);
   $ENV{HPCALE_SERVER_HOME} = $libs[0];

   chomp($libs[1]);
   my @pmarray = split /:/ , $libs[1];

   for(2..13){
      chomp($libs[$_]);
   }

   $ENV{HPCALE_LOGPATH} = $libs[2];
   $ENV{HPCALE_LOGFILE} = $libs[3];
   $ENV{HPCALE_DEBUG_LEVEL} = $libs[4];
   $ENV{HPCALE_JOBDIR} = $libs[5];
   $ENV{HPCALE_REMOTE_JOBDIR} = $libs[6];
   $ENV{HPCALE_RESERVE_RETRY} = $libs[7];
   $ENV{HPCALE_SERVER_HOST} = $libs[8];
   $ENV{HPCALE_SERVER_DB_HOST} = $libs[9];
   $ENV{HPCALE_SERVER_DB_PORT} = $libs[10];
   $ENV{HPCALE_SERVER_DB} = $libs[11];
   $ENV{HPCALE_SERVER_DB_USER} = $libs[12];
   $ENV{HPCALE_SERVER_DB_PASS} = $libs[13];
   
   foreach(@pmarray){
      push (@INC, $_);
   }
   push (@INC, "$ENV{HPCALE_SERVER_HOME}");
   push (@INC, "$ENV{HPCALE_SERVER_HOME}/server");
}

use SOAP::Transport::HTTP;

SOAP::Transport::HTTP::CGI
    -> dispatch_to('soapsvc')
    -> handle;


package soapsvc;

use util::Debug;
use util::XJDParser;
use allocation::RC_Locker;
use execution::ComponentExecute;
use runtimeenv::RTEnvAnalyze;
use notification::JobTerminationNotifier;
use strict;

my $mod = undef;
my $pipe= undef;

sub stopComponents {
   my ($self, $user, $chost, $jobid, $clienturi, $clientproxy) = @_;

   $mod = "JobTerminationService(CGI)";
   $pipe= "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";

   Debug::dbgmsg(1, $mod, $pipe, "========= JOB TERMINATION =========");
   Debug::dbgmsg(2, $mod, $pipe, "CLIENT URI: $clienturi");
   Debug::dbgmsg(2, $mod, $pipe, "CLIENT PROXY: $clientproxy");
   Debug::dbgmsg(2, $mod, $pipe, "JobID: $jobid");

   my $notifier = new JobTerminationNotifier($clienturi, $clientproxy);

   my $child_pid;        
   if (!defined($child_pid = fork())) {
      Debug::logmsg($mod, $pipe, "Can't fork");
      notifier->forkFailed ;
      return -1;
   } 
   # this is parent (pid > 0)
   elsif ($child_pid) {
      # just let the client know fork is ok. Refer svc_discover_rc.pl
      $notifier->forkOk($child_pid);
      return 1;
   }
   # this is child (pid 0)
   # for more info, refer svc_discover_rc.pl
   else {
      if($notifier->forkOk (0) < 0){
         Debug::logmsg($mod, $pipe, "Communication Error with client daemon");
      }
      else{
         #
         # Obtain XJD full name.
         # Assume that there is only one XJD file under job directory)
         #
         my @xjdfile = glob("$ENV{HPCALE_JOBDIR}/$jobid/*.xjd");

         if(scalar @xjdfile < 1){
            Debug::errmsg($mod, $pipe, "No XJD under job directory");
            $self->notify_result($notifier, -1);
            return;
         }
         Debug::logmsg($mod, $pipe, "XJD to reorganize: $xjdfile[0]");
         my $lowXJD = new XJDParser($xjdfile[0]);

         #
         # reorganize the info generated at rtenv phase based on the 
         # output generated by runtime environment setup phase.
         #
         my $rtenvAnalyzer = new RTEnvAnalyze();
         my $reorganized = $rtenvAnalyzer->reorganize($user, $chost, $jobid, $lowXJD);
         if($reorganized < 0){
            Debug::errmsg($mod, $pipe, "Reorganizing runtime environment info: failed.");
            # job terminator does not have right to release the resources
            # if it cannot reorganize the runtime information
            #(new RC_Locker())->removeLocks($ENV{HPCALE_JOBDIR}, $jobid);
            $self->notify_result($notifier, -1);
            return;
         }
         else{
            Debug::logmsg($mod, $pipe, "Reorganizing runtime environment info: ok");
         }

         #
         # force components to stop on the corresponding resources 
         #
         my $compExecutor = new ComponentExecute($jobid);   
         my $result = $compExecutor->stopExecution($lowXJD);

         # In either case, we cleanup the resources
         $rtenvAnalyzer->cleanupRTenv;
         (new RC_Locker())->removeLocks($ENV{HPCALE_JOBDIR}, $jobid);

         if($result < 0){
            Debug::errmsg($mod, $pipe, "Job termination Failed. HPCALE tries to release resources");

            # if failure, let client know this sad news. :)
            $self->notify_result($notifier, -1);
            return;
         }
         else{
            Debug::logmsg($mod, $pipe, "Job termination finished well.");

            # if success, let client know this happy news. :)
            $self->notify_result($notifier, 1);
         } 
      }

      return; # just to finish here
   }

   Debug::logmsg($mod, $pipe, "Unknown Error");
   return -1;
}

sub notify_result
{
   my ($self, $notifier, $flag) = @_;

   if($notifier->notify($flag) == 1){
      Debug::logmsg($mod, $pipe, "client notification: ok");
   }
   else{
      Debug::logmsg($mod, $pipe, "client notification: failed");
   }
}


