# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package Debug;

use strict;
use Time::Local;
use Data::Dumper;

my $HPCALE_DEBUG_LEVEL = $ENV{HPCALE_DEBUG_LEVEL};

sub dbgmsg{
   my $level = shift;
   my $mod   = shift;
   my $pipe  = shift;
   my $msg   = shift;

   my $stream = undef;
   my $tstamp = &create_timestamp ;

   if ($pipe eq "STDOUT"){
      print STDOUT "$tstamp:\t DBG_$level: $mod: $msg\n" if ($level<=$HPCALE_DEBUG_LEVEL);
   }
   elsif($pipe eq "STDERR"){
      print STDERR "$tstamp:\tDBG_$level: $mod: $msg\n" if ($level<=$HPCALE_DEBUG_LEVEL);
   }
   else{
      if(-e $pipe){
         open($stream, ">> $pipe"); 
      }
      else{
         open($stream, "> $pipe");
      }

      print $stream "$tstamp:\tDBG_$level: $mod: $msg\n" if ($level<=$HPCALE_DEBUG_LEVEL);
      close($stream);
   }
}

sub logmsg{
   my $mod   = shift;
   my $pipe  = shift;
   my $msg   = shift;

   my $stream = undef;
   my $tstamp = &create_timestamp ;

   if ($pipe eq "STDOUT"){
      print STDOUT "$tstamp:LOG: $mod: $msg\n";
   }
   elsif($pipe eq "STDERR"){
      print STDERR "$tstamp:LOG: $mod: $msg\n";
   }
   else{
      #open($stream, "> $pipe"); 
      if(-e $pipe){
         open($stream, ">> $pipe"); 
      }
      else{
         open($stream, "> $pipe");
      }

      print $stream "$tstamp:LOG: $mod: $msg\n";
      close($stream);
   }
}

# We can handle error message separately to the log message
# if necessary in the future. 
sub errmsg{
   my $mod   = shift;
   my $pipe  = shift;
   my $msg   = shift;

   my $stream = undef;
   my $tstamp = &create_timestamp ;

   if ($pipe eq "STDOUT"){
      print STDOUT "$tstamp:ERR: $mod: $msg\n";
   }
   elsif($pipe eq "STDERR"){
      print STDERR "$tstamp:ERR: $mod: $msg\n";
   }
   else{
      if(-e $pipe){
         open($stream, ">> $pipe"); 
      }
      else{
         open($stream, "> $pipe");
      }

      print $stream "$tstamp:ERR: $mod: $msg\n";
      close($stream);
   }
}

sub create_timestamp {
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);

    $year += 1900;
    $mon++;
    return "$year/$mon/$mday $hour:$min:$sec";
}



1;
