# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package RemoteDirManager;

BEGIN{
    push (@INC, "$ENV{HPCALE_SERVER_HOME}");
}

use util::Debug;
use strict;

my $mod = "RemoteDirManager";
my $pipe= "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";

my $ERR000 = "Directory is created.";
my $ERR001 = "Directory alreay exists.";
my $ERR002 = "Host is unavailable.";

sub new {
   my $classname  = shift;         # What class are we constructing?
   my $self   = {};                # Allocate new memory
   bless($self, $classname);       # Mark it of the right type
   $self->{msg} = "";
   $self->{answer} = undef;
   return $self;
}

#
# Return -1 if fail to create
# Return -2 if duplicate directory
# Otherwise, return 1
#
sub createRemoteDir {
   my ($self,$host,$path) = @_;

   my $hostok = $self->isHostAlive($host) ;
   my $home = "";
   my $direxist = -1;

   if($hostok < 0){
      Debug::errmsg($mod, $pipe, $ERR002);
      $self->{answer} = -1;

      return $self->{answer};
   }
   else{
      $home = $self->getHomeDir($host);
      $direxist = $self->dirExist($host, "$home/$path");
   }

   if($direxist == 1){
      Debug::logmsg($mod, $pipe, "$ERR001: $home/$path");
      $self->{answer} = -2;
   }
   else{
      Debug::dbgmsg(3, $mod, $pipe, "Creating Directory <$home/$path>...");

      my $r = system "ssh $host '/bin/sh -c \"mkdir -p $home/$path 2>/dev/null\"'";
      if ($r == 0){
         $self->{answer} = 1;
      }
      else{
         $self->{answer} = -1;
      }
   }

   return $self->{answer};
}

sub dirExist {
   my $self = shift;
   my $host = shift;
   my $path = shift;

   my $r = system "ssh $host '/bin/sh -c \"ls $path 2>/dev/null\"'";
   ($r == 0) ? return 1 : return -1;
}

sub fileExist {
   my $self = shift;
   my $host = shift;
   my $file = shift;

   my $r = system "ssh $host '/bin/sh -c \"ls $file 2>/dev/null\"'";
   ($r == 0) ? return 1 : return -1;
}

sub getHomeDir {
   my $self = shift;
   my $host = shift;

   # get home directory of clusterheads[$i]
   my $home = `ssh $host 'env | grep '^HOME=''`;
   $home =~ /.*=(.*)/;
   $home = $1;
   chomp $home;

   Debug::dbgmsg(3, $mod, $pipe, "Home of $host: $home");
   return $home;
}

sub checkProgramDir {
   my $self = shift;
   my $host = shift;
   my $prog = shift;

   # get home directory of clusterheads[$i]
   my $loc = `ssh $host 'which $prog '`;
   chomp $loc;

   Debug::dbgmsg(3, $mod, $pipe, "Remote location for program $prog is $loc");
   return $loc;
}


sub isHostAlive {
   my $self = shift;
   my $host = shift;

   my $id = `ssh $host 'whoami'`;

   if(length($id) == 0){
      Debug::dbgmsg(3, $mod, $pipe, "$host: dead.");
      return -1;
   }
   else{
      Debug::dbgmsg(3, $mod, $pipe, "$host: alive.");
      return 1;
   }
}

1;
